from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeStitchAction(DbMetadataEdge):
    NAME = 'Edge - STITCH - Gene Drug (Action)'
    URL = "http://stitch.embl.de/download/actions.v5.0/9606.actions.v5.0.tsv.gz"
    OFILE_NAME = "STITCH_gene_drug_actions.tsv.gz"
    COLS = ['item_id_a', 'item_id_b', 'mode', 'action', 'a_is_acting', 'score']
    FILTER_COLS = ['item_id_a', 'item_id_b', 'mode', 'action', 'score']
    HEADER = 1
    DB_TYPE = DbType.DB_EDGE_STITCH_ACTION

    def __init__(self):
        super().__init__(url=DbMetaEdgeStitchAction.URL,
                         ofile_name=DbMetaEdgeStitchAction.OFILE_NAME,
                         dbType=DbMetaEdgeStitchAction.DB_TYPE)



from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeBgeeDiffExpr import DbMetaEdgeBgeeDiffExpr
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeBgeeExpr import DbMetaEdgeBgeeExpr
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeCtdPath import DbMetaEdgeCtdPath
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeDisGeNet import DbMetaEdgeDisGeNet
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeDrugCentral import DbMetaEdgeDrugCentral
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeGo import DbMetaEdgeGo
# from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeHpa import        DbMetaEdgeHpa
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeHpoDis import DbMetaEdgeHpoDis
# from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeSiderInd import   DbMetaEdgeSiderInd
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeHpoGene import DbMetaEdgeHpoGene
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeSiderSe import DbMetaEdgeSiderSe
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeStitch import DbMetaEdgeStitch
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeStitchAction import DbMetaEdgeStitchAction
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeString import DbMetaEdgeString
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeStringAction import DbMetaEdgeStringAction
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeTnHpoDis import DbMetaEdgeTnHpoDis
from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge



from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeStitch(DbMetadataEdge):
    NAME = 'Edge - STITCH - Gene Drug'
    URL = "http://stitch.embl.de/download/protein_chemical.links.v5.0/9606.protein_chemical.links.v5.0.tsv.gz"
    OFILE_NAME = "STITCH_gene_drug.tsv.gz"
    COLS = ['chemID', 'stringID', 'qscore']
    FILTER_COLS = ['stringID', 'chemID', 'qscore']
    HEADER = 1
    DB_TYPE = DbType.DB_EDGE_STITCH

    def __init__(self):
        super().__init__(url=DbMetaEdgeStitch.URL,
                         ofile_name=DbMetaEdgeStitch.OFILE_NAME,
                         dbType=DbMetaEdgeStitch.DB_TYPE)



from openbiolink.graph_creation.metadata_edge.edgeMetadata import EdgeMetadata
from openbiolink.graph_creation.types.qualityType import QualityType


class EdgeRegularMetadata(EdgeMetadata):
    LQ_CUTOFF = None
    MQ_CUTOFF = None
    HQ_CUTOFF = None
    LQ_CUTOFF_TEXT = None
    MQ_CUTOFF_TEXT = None
    HQ_CUTOFF_TEXT = None

    def __init__(self, is_directional, edges_file_path, colindex1, colindex2, edgeType, node1_type, node2_type,
                 colindex_qscore=None,
                 mapping1_file=None, map1_sourceindex=None, map1_targetindex=None,
                 altid_mapping1_file=None, altid_map1_sourceindex=None, altid_map1_targetindex=None,
                 mapping2_file=None, map2_sourceindex=None, map2_targetindex=None,
                 altid_mapping2_file=None, altid_map2_sourceindex=None, altid_map2_targetindex=None,
                 quality: QualityType = None):
        if quality is QualityType.HQ:
            cutoff_txt = self.HQ_CUTOFF_TEXT
            cutoff_num = self.HQ_CUTOFF
        elif quality is QualityType.MQ:
            cutoff_txt = self.MQ_CUTOFF_TEXT
            cutoff_num = self.MQ_CUTOFF
        elif quality is QualityType.LQ:
            cutoff_txt = self.LQ_CUTOFF_TEXT
            cutoff_num = self.LQ_CUTOFF
        else:
            cutoff_txt = None
            cutoff_num = None
        super().__init__(is_directional=is_directional,
                         edges_file_path=edges_file_path,
                         colindex1=colindex1, colindex2=colindex2,
                         edgeType=edgeType,
                         node1_type=node1_type, node2_type=node2_type,
                         colindex_qscore=colindex_qscore, cutoff_num=cutoff_num, cutoff_txt=cutoff_txt,
                         mapping1_file=mapping1_file,
                         map1_sourceindex=map1_sourceindex, map1_targetindex=map1_targetindex,
                         altid_mapping1_file=altid_mapping1_file,
                         altid_map1_sourceindex=altid_map1_sourceindex, altid_map1_targetindex=altid_map1_targetindex,
                         mapping2_file=mapping2_file,
                         map2_sourceindex=map2_sourceindex, map2_targetindex=map2_targetindex,
                         altid_mapping2_file=altid_mapping2_file,
                         altid_map2_sourceindex=altid_map2_sourceindex, altid_map2_targetindex=altid_map2_targetindex
                         )



from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeSiderSe(DbMetadataEdge):
    NAME = 'Edge - Sider - Side Effects'
    URL = "http://sideeffects.embl.de/media/download/meddra_all_se.tsv.gz"
    OFILE_NAME = "SIDER_se.tsv.gz"
    COLS = ['stitchID_flat', 'stitchID_stereo', 'umlsID', 'medDRAumlsType', 'medDRAumlsID', 'SEname']
    FILTER_COLS = ['stitchID_stereo', 'umlsID']
    HEADER = 0
    DB_TYPE = DbType.DB_EDGE_SIDER_SE

    def __init__(self):
        super().__init__(url=DbMetaEdgeSiderSe.URL,
                         ofile_name=DbMetaEdgeSiderSe.OFILE_NAME,
                         dbType=DbMetaEdgeSiderSe.DB_TYPE)



DEFAULT_HITS_AT_K = [1, 5, 10, 30, 50]
EVAL_OUTPUT_FOLDER_NAME = 'evaluation'
EVAL_OUTPUT_FILE_NAME = 'evaluation_result.txt'
CORRUPTED_HEADS_FILE_NAME = 'corrupted_heads.csv'
CORRUPTED_TAILS_FILE_NAME = 'corrupted_tails.csv'
CORRUPTED_GROUP_COL_NAME = 'number'

MODEL_DIR = 'model'
MODEL_TRAINED_NAME = 'trained_model.pkl'
MODEL_ENTITY_NAME_MAPPING_NAME = 'entity_name_to_id.json.json'
MODEL_ENTITY_TYPE_MAPPING_NAME = 'entity_type_to_id.json'
MODEL_RELATION_TYPE_MAPPING_NAME = 'relation_type_to_id.json'



import setuptools

with open("../README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="openbiolink", 
	version="0.1.01",
	author="Anna Breit, Matthias Samwald, Simon Ott, Laura Graf, Asan Agibetov",
	author_email="matthiassamwald@gmail.com",
	description=" A framework for evaluating link prediction models on heterogeneous biomedical graph data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/OpenBioLink/OpenBioLink",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
	install_requires=[
		'numpy',
		'pandas==0.23.4',
		'pykeen',
		'pytest==5.0.1',
		'scikit-learn==0.19.1; python_version == "3.6"',
		'scikit-learn; python_version == "3.7"',
		'tqdm==4.29.1',
		'sortedcontainers',
	],
    python_requires='>=3.6',
)


from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapDoAltidProcessor import OntoMapDoAltidProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapDoOmimProcessor import OntoMapDoUmlsProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapDoUmlsProcessor import OntoMapDoUmlsProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapGoAltidProcessor import OntoMapGoAltidProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapHpoAltidProcessor import OntoMapHpoAltidProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapHpoUmlsProcessor import OntoMapHpoUmlsProcessor
from openbiolink.graph_creation.file_processor.onto_mapping.ontoMapUberonAltidProcessor import OntoMapUberonAltidProcessor



from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeHpa(DbMetadataEdge):
    NAME = 'Edge - HPA - Expression Data'
    URL = "https://www.proteinatlas.org/download/rna_tissue.tsv.zip"
    OFILE_NAME = "HPA_gene_anatomy.tsv.zip"
    COLS = ['geneID', 'geneName', 'anatomy', 'expressionValue', 'Unit']
    FILTER_COLS = ['geneID', 'anatomy', 'expressionValue']
    HEADER = 1
    DB_TYPE = DbType.DB_EDGE_HPA

    def __init__(self):
        super().__init__(url=DbMetaEdgeHpa.URL,
                         ofile_name=DbMetaEdgeHpa.OFILE_NAME,
                         dbType=DbMetaEdgeHpa.DB_TYPE)



import os

from openbiolink.graph_creation import graphCreationConfig as g
# from openbiolink.graph_creationtypes.dbType import DbType
# from openbiolink.graph_creationtypes.readerType import ReaderType
from openbiolink.graph_creation.file_reader.postgresDumpReader import PostgresDumpReader


class MapDrugCentralPubchemReader(PostgresDumpReader):
    """ to create a new csv file reader:
              *) declare the corresponding DB_META_CLASS, DB_META_CLASS.table_name, DB_META_CLASS.cols, readerType, as well as dbType
              *) for clearer structure, move class to corresponding module (and import in corresponding init)
              prior steps necessary:
              *) create DB_META_CLASS
              *) add readerType
              *) add dbType
        """

    DB_META_CLASS = None  # database meta class here

    def __init__(self):
        super().__init__(
            in_path=os.path.join(g.O_FILE_PATH, self.DB_META_CLASS.OFILE_NAME),
            table_name=None,  # table defined in DB_META_CLASS here
            cols=None,  # cols defined in DB_META_CLASS here
            readerType=None,  # reader type here
            dbType=None  # database type here
        )



import os

from openbiolink.graph_creation import graphCreationConfig as glob
from openbiolink.graph_creation.metadata_edge.edgeRegularMetadata import EdgeRegularMetadata
from openbiolink.graph_creation.metadata_infile.edge.inMetaEdgeCdtPath import InMetaEdgeCdtPath
from openbiolink.graph_creation.types.qualityType import QualityType


class EdgeMetaGenePath(EdgeRegularMetadata):
    NAME = 'Edge - Gene_association_Pathway'  # todo ms wording

    EDGE_INMETA_CLASS = InMetaEdgeCdtPath

    def __init__(self, quality: QualityType):
        edges_file_path = os.path.join(glob.IN_FILE_PATH, self.EDGE_INMETA_CLASS.CSV_NAME)
        super().__init__(is_directional=True,
                         edges_file_path=edges_file_path,
                         colindex1=self.EDGE_INMETA_CLASS.NODE1_COL, colindex2=self.EDGE_INMETA_CLASS.NODE2_COL,
                         edgeType=self.EDGE_INMETA_CLASS.EDGE_TYPE,
                         node1_type=self.EDGE_INMETA_CLASS.NODE1_TYPE, node2_type=self.EDGE_INMETA_CLASS.NODE2_TYPE,
                         colindex_qscore=self.EDGE_INMETA_CLASS.QSCORE_COL)



from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeGo(DbMetadataEdge):
    NAME = 'Edge - GO - GO Annotations'
    URL = "http://geneontology.org/gene-associations/goa_human.gaf.gz"
    OFILE_NAME = "GO_annotations.gaf.gz"
    COLS = ['DB', 'DOI', 'DB_symbol', 'qulifier', 'GO_ID', 'DB_ref', 'evidence_code',
            'with_from', 'aspect', 'DB_obj_name', 'DB_obj_syn', 'DB_obj_type', 'taxon', 'date', 'assigned_by',
            'ann_ext', 'ann_prop']
    FILTER_COLS = ['DOI', 'GO_ID', 'evidence_code']
    HEADER = 30
    DB_TYPE = DbType.DB_EDGE_GO

    def __init__(self):
        super().__init__(url=DbMetaEdgeGo.URL,
                         ofile_name=DbMetaEdgeGo.OFILE_NAME,
                         dbType=DbMetaEdgeGo.DB_TYPE)



from openbiolink.graph_creation.metadata_infile.infileMetadata import InfileMetadata
from openbiolink.graph_creation.types.infileType import InfileType


class InMetaMapDisGeNet(InfileMetadata):
    CSV_NAME = "DB_DisGeNet_mapping_disease_umls_do.csv"
    USE_COLS = ['umlsID', 'code']  # voc gets deleted while individual preprocessing
    SOURCE_COL = 0
    TARGET_COL = 1
    MAPPING_SEP = None
    INFILE_TYPE = InfileType.IN_MAP_DISGENET

    def __init__(self):
        super().__init__(csv_name=InMetaMapDisGeNet.CSV_NAME,
                         cols=self.USE_COLS,
                         infileType=InMetaMapDisGeNet.INFILE_TYPE)



from openbiolink.graph_creation.file_processor.fileProcessor import FileProcessor
from openbiolink.graph_creation.metadata_infile.onto.inMetaOntoGoIsA import InMetaOntoGoIsA
from openbiolink.graph_creation.types.infileType import InfileType
from openbiolink.graph_creation.types.readerType import ReaderType


class OntoGoIsAProcessor(FileProcessor):
    IN_META_CLASS = InMetaOntoGoIsA

    def __init__(self):
        self.use_cols = self.IN_META_CLASS.USE_COLS
        super().__init__(self.use_cols, readerType=ReaderType.READER_ONTO_GO,
                         infileType=InfileType.IN_ONTO_GO_IS_A, mapping_sep=self.IN_META_CLASS.MAPPING_SEP)



# from openbiolink.graph_creation.types.infileType import InfileType
# from openbiolink.graph_creation.types.readerType import ReaderType
from openbiolink.graph_creation.file_processor.fileProcessor import FileProcessor


class MyFileProcessor(FileProcessor):
    """ to create a new file processor
    *) declare the corresponding META_EDGE_CLASS, readerType, as well as infileType
    *) add pre- or post-processing steps if necessary
    *) for clearer structure, move class to corresponding module (and import in corresponding init)
    prior steps necessary:
    *) create META_EDGE_CLASS
    *) add readerType
    *) add infileType
     """

    IN_META_CLASS = None  # meta infile here

    def __init__(self):
        self.use_cols = self.IN_META_CLASS.USE_COLS
        super().__init__(self.use_cols,
                         readerType=None,  # reader type here
                         infileType=None,  # infile type here
                         mapping_sep=self.IN_META_CLASS.MAPPING_SEP)

    # def individual_preprocessing(self, data):
    """ data processing before selection of used columns """
    #    return data

    # def individual_postprocessing(self, data):
    """ data processing after selection of used columns """
    #    return data



import os

from openbiolink.graph_creation import graphCreationConfig as g
from openbiolink.graph_creation.file_reader.postgresDumpReader import PostgresDumpReader
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeDrugCentral import DbMetaEdgeDrugCentral
from openbiolink.graph_creation.types.dbType import DbType
from openbiolink.graph_creation.types.readerType import ReaderType


class MapDrugCentralPubchemReader(PostgresDumpReader):
    DB_META_CLASS = DbMetaEdgeDrugCentral

    def __init__(self):
        super().__init__(
            in_path=os.path.join(g.O_FILE_PATH, self.DB_META_CLASS.OFILE_NAME),
            table_name=self.DB_META_CLASS.TABLE_NAME_MAP_PUBCHEM,
            cols=self.DB_META_CLASS.COLS_MAP_PUBCHEM,
            readerType=ReaderType.READER_MAP_DRUGCENTRAL_PUBCHEM,
            dbType=DbType.DB_EDGE_DRUGCENTRAL
        )



from openbiolink.graph_creation.metadata_infile.edge import *
from openbiolink.graph_creation.metadata_infile.infileMetadata import InfileMetadata
from openbiolink.graph_creation.metadata_infile.mapping import *
from openbiolink.graph_creation.metadata_infile.onto import *



from enum import Enum

import openbiolink.evaluation.models.pykeen_models as pykeen_models


class ModelTypes(Enum):
    # PYKEEN_TRANS_E = 'TransE_PyKeen'
    # PYKEEN_TRANS_R = 'TransR_PyKeen'
    TransE_Pykeen = pykeen_models.TransE_PyKeen
    TransR_Pykeen = pykeen_models.TransR_PyKeen



from enum import Enum


class InfileType(Enum):
    IN_ONTO_GO_IS_A = 0
    IN_ONTO_DO_IS_A = 1
    IN_ONTO_HPO_IS_A = 2
    IN_ONTO_UBERON_IS_A = 3
    IN_ONTO_GO_PART_OF = 4
    IN_ONTO_UBERON_PART_OF = 5

    IN_EDGE_CDT_PATH = 100
    IN_EDGE_DISGENET = 101
    IN_EDGE_GO = 102
    IN_EDGE_HPA = 103
    IN_EDGE_HPO_DIS = 104
    IN_EDGE_HPO_GENE = 105
    IN_EDGE_SIDER_IND = 106
    IN_EDGE_SIDER_SE = 107
    IN_EDGE_STITCH = 108
    IN_EDGE_STRING = 109
    IN_EDGE_DRUGCENTRAL_IND = 110
    IN_EDGE_BGEE_EXPR = 111
    IN_EDGE_BGEE_NO_EXPR = 112
    IN_EDGE_BGEE_OVEREXPR = 113
    IN_EDGE_BGEE_UNDEREXPR = 114
    IN_EDGE_STITCH_ACTIVATION = 115
    IN_EDGE_STITCH_BINDING = 116
    IN_EDGE_STITCH_CATALYSIS = 117
    IN_EDGE_STITCH_EXPRESSION = 118
    IN_EDGE_STITCH_INHIBITION = 119
    IN_EDGE_STITCH_PREDBIND = 120
    IN_EDGE_STITCH_REACTION = 121
    IN_EDGE_STITCH_BINDACT = 122
    IN_EDGE_STITCH_BINDINH = 123
    IN_EDGE_TN_HPO_DIS = 124

    IN_EDGE_STRING_ACTIVATION = 125
    IN_EDGE_STRING_BINDING = 126
    IN_EDGE_STRING_CATALYSIS = 127
    IN_EDGE_STRING_EXPRESSION = 128
    IN_EDGE_STRING_INHIBITION = 129
    IN_EDGE_STRING_PTMOD = 130
    IN_EDGE_STRING_REACTION = 131
    IN_EDGE_STRING_BINDACT = 132
    IN_EDGE_STRING_BINDINH = 133

    IN_EDGE_DRUGCENTRAL_CONTRA_IND = 150

    IN_MAP_DISGENET = 200
    IN_MAP_STRING = 201
    IN_MAP_UNI_UNI_NCBI = 202
    IN_MAP_UNI_ENS_NCBI = 203
    IN_MAP_DRUGCENTRAL_PUBCHEM = 204

    IN_MAP_ONTO_DO_UMLS = 301
    IN_MAP_ONTO_DO_OMIM = 302
    IN_MAP_ONTO_HPO_UMLS = 303

    IN_MAP_ONTO_DO_ALT_ID = 400
    IN_MAP_ONTO_GO_ALT_ID = 401
    IN_MAP_ONTO_HPO_ALT_ID = 402
    IN_MAP_ONTO_UBERON_ALT_ID = 403



from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeDisGeNet(DbMetadataEdge):
    NAME = 'Edge - DisGeNet - Gene Disease'
    # URL = "http://www.disgenet.org/ds/DisGeNET/results/curated_gene_disease_associations.tsv.gz"
    URL = "http://www.disgenet.org/static/disgenet_ap1/files/downloads/curated_gene_disease_associations.tsv.gz"
    OFILE_NAME = "DisGeNet_gene_disease.tsv.gz"
    COLS = ['geneID', 'geneSym', 'DSI', 'DPI', 'umlsID', 'disName', 'diseaseType', 'diseaseClass',
            'diseaseSemanticType', 'score', 'EI', 'YearInitial', 'YearFinal', 'NofPmids', 'NofSnps', 'source']

    FILTER_COLS = ['geneID', 'umlsID', 'score']
    HEADER = 1
    DB_TYPE = DbType.DB_EDGE_DISGENET

    def __init__(self):
        super().__init__(url=DbMetaEdgeDisGeNet.URL,
                         ofile_name=DbMetaEdgeDisGeNet.OFILE_NAME,
                         dbType=DbMetaEdgeDisGeNet.DB_TYPE)



from openbiolink.graph_creation.file_reader.onto.ontoDoReader import OntoDoReader
from openbiolink.graph_creation.file_reader.onto.ontoGoReader import OntoGoReader
from openbiolink.graph_creation.file_reader.onto.ontoHpoReader import OntoHpoReader
from openbiolink.graph_creation.file_reader.onto.ontoUberonReader import OntoUberonReader



from openbiolink.graph_creation.file_processor.edge.edgeBgeeExprProcessor import EdgeBgeeExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeBgeeNoExprProcessor import EdgeBgeeNoExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeBgeeOverExprProcessor import EdgeBgeeOverExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeBgeeUnderExprProcessor import EdgeBgeeUnderExprProcessor
from openbiolink.graph_creation.file_processor.edge.edgeCdtPathProcessor import EdgeCdtPathProcessor
from openbiolink.graph_creation.file_processor.edge.edgeDisGeNetProcessor import EdgeDisGeNetProcessor
from openbiolink.graph_creation.file_processor.edge.edgeDrugCentralContraIndProcessor import EdgeDrugCentralIndProcessor
from openbiolink.graph_creation.file_processor.edge.edgeDrugCentralIndProcessor import EdgeDrugCentralIndProcessor
from openbiolink.graph_creation.file_processor.edge.edgeGoProcessor import EdgeGoProcessor
#from openbiolink.graph_creation.file_processor.edge.edgeHpaProcessor import EdgeHpaProcessor --> replaced by bgee
from openbiolink.graph_creation.file_processor.edge.edgeHpoDisProcessor import EdgeHpoDisProcessor
from openbiolink.graph_creation.file_processor.edge.edgeHpoGeneProcessor import EdgeHpoGeneProcessor
# from openbiolink.graph_creation.file_processor.edge.edgeSiderIndProcessor import EdgeSiderIndProcessor --> replaced by drugcentral
from openbiolink.graph_creation.file_processor.edge.edgeSiderSeProcessor import EdgeSiderSeProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchActivationProcessor import EdgeStitchActivationProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchBindActProcessor import EdgeStitchBindActProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchBindInhProcessor import EdgeStitchBindInhProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchBindingProcessor import EdgeStitchBindingProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchCatalysisProcessor import EdgeStitchCatalysisProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchExpressionProessor import EdgeStitchExpressionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchInhibitionProcessor import EdgeStitchInhibitionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchPredBindProcessor import EdgeStitchPredBindProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchProcessor import EdgeStitchProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStitchReactionProcessor import EdgeStitchReactionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringActivationProcessor import EdgeStringActivationProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringBindActProcessor import EdgeStringBindActProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringBindInhProcessor import EdgeStringBindInhProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringBindingProcessor import EdgeStringBindingProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringCatalysisProcessor import EdgeStringCatalysisProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringExpressionProcessor import EdgeStringExpressionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringInhibitionProcessor import EdgeStringInhibitionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringProcessor import EdgeStringProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringPtmodeProcessor import EdgeStringPtmodeProcessor
from openbiolink.graph_creation.file_processor.edge.edgeStringReactionProcessor import EdgeStringReactionProcessor
from openbiolink.graph_creation.file_processor.edge.edgeTnHpoDisProcessor import EdgeTnHpoDisProcessor



import logging
import sys

import numpy as np

from openbiolink import globalConfig
from openbiolink import globalConfig as globConst
from openbiolink.cli import Cli
from openbiolink.graph_creation.file_reader.fileReader import FileReader
from openbiolink.graph_creation.file_reader.parser.oboParser import OboParser


class OboReader(FileReader):

    def __init__(self, in_path, readerType, dbType, quadruple_list=None):
        ''' quadruple_list should contain all quadruples incl. mappings
        each quadruple consists of
        (1) beginning of the line,
        (2) split character,
        (3) index of split element being the id,
        (4) the name of dict entry (must be same as in use_fields)'''

        super().__init__(in_path, readerType, dbType)
        if quadruple_list is None:
            self.quadruple_list = [('id', ' ', 1, 'ID'),
                                   ('alt_id', ' ', 1, 'ALT_ID'),
                                   ('is_a', ' ', 1, 'IS_A'),
                                   ('xref: UMLS:', ':', 2, 'UMLS'),
                                   ('xref: OMIM:', ' ', 1, 'OMIM')]
        else:
            self.quadruple_list = quadruple_list

    def read_file(self):
        oboParser = OboParser()
        with FileReader.open_file(self.in_path) as file:
            df = oboParser.obo_to_df(file, self.quadruple_list)
            df_cols = df.columns
            defined_cols = [x[3] for x in self.quadruple_list]
            if len(df_cols) != len(defined_cols):

                no_occurences = [x for x in defined_cols if x not in df_cols]
                info_string = 'Reader %s should parse %s but there are no occurrences in file %s. ' % (
                str(self.readerType), str(no_occurences), self.in_path)
                if globalConfig.INTERACTIVE_MODE:
                    ask_continue_string = info_string + 'Continue if you do not need these edges in your graph'
                    if globConst.GUI_MODE:
                        from openbiolink.gui import gui
                        gui.askForExit(ask_continue_string)
                    else:
                        Cli.ask_for_exit(ask_continue_string)
                    for col in no_occurences:
                        df[col] = np.nan
                else:
                    logging.error(info_string)
                    sys.exit(info_string)

            return df



from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeTnHpoDis(DbMetadataEdge):
    NAME = 'Edge - HPO - Disease Phenotype (Negative)'

    URL = "http://compbio.charite.de/jenkins/job/hpo.annotations/lastStableBuild/artifact/misc/negative_phenotype_annotation.tab"
    OFILE_NAME = "HPO_TN_disease_phenotype.tab"
    COLS = ['DB', 'DOI', 'DBname', 'qulifier', 'HPO_ID', 'DB_ref',
            'evidence_code', 'onsetMod', 'freq', 'sex',
            'mod', 'aspect', 'date', 'assigned_by']
    FILTER_COLS = ['DB_ref', 'HPO_ID', 'evidence_code']
    HEADER = 0
    DB_TYPE = DbType.DB_EDGE_TN_HPO_DIS

    def __init__(self):
        super().__init__(url=DbMetaEdgeTnHpoDis.URL,
                         ofile_name=DbMetaEdgeTnHpoDis.OFILE_NAME,
                         dbType=DbMetaEdgeTnHpoDis.DB_TYPE)



import os

from openbiolink.graph_creation import graphCreationConfig as g
from openbiolink.graph_creation.file_reader.csvReader import CsvReader
from openbiolink.graph_creation.metadata_db_file.edge.dbMetaEdgeBgeeExpr import DbMetaEdgeBgeeExpr
from openbiolink.graph_creation.types.dbType import DbType
from openbiolink.graph_creation.types.readerType import ReaderType


class EdgeBgeeReader(CsvReader):
    DB_META_CLASS = DbMetaEdgeBgeeExpr

    def __init__(self):
        super().__init__(
            in_path=os.path.join(g.O_FILE_PATH, self.DB_META_CLASS.OFILE_NAME),
            sep=None,
            cols=self.DB_META_CLASS.COLS,
            use_cols=self.DB_META_CLASS.FILTER_COLS,
            nr_lines_header=self.DB_META_CLASS.HEADER,
            readerType=ReaderType.READER_EDGE_BGEE,
            dbType=DbType.DB_EDGE_BGEE
        )



from openbiolink.graph_creation.metadata_db_file.edge.dbMetadataEdge import DbMetadataEdge
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaEdgeBgeeExpr(DbMetadataEdge):
    NAME = 'Edge - Bgee - presence/absence of expression'
    URL = "ftp://ftp.bgee.org/current/download/calls/expr_calls/Homo_sapiens_expr_simple.tsv.gz"
    OFILE_NAME = "BGEE_expr_calls.tsv.gz"

    COLS = ['gene_id', 'gene_name', 'anatomical_entity', 'anatomical_entity_name',
            'expression', 'call_quality', 'expression_rank']
    FILTER_COLS = ['gene_id', 'anatomical_entity', 'expression', 'call_quality']
    HEADER = 1
    DB_TYPE = DbType.DB_EDGE_BGEE

    def __init__(self):
        super().__init__(url=DbMetaEdgeBgeeExpr.URL,
                         ofile_name=DbMetaEdgeBgeeExpr.OFILE_NAME,
                         dbType=DbMetaEdgeBgeeExpr.DB_TYPE)



from openbiolink.graph_creation.metadata_db_file.onto.dbMetadataOnto import DbMetadataOnto
from openbiolink.graph_creation.types.dbType import DbType


class DbMetaOntoDo(DbMetadataOnto):
    NAME = 'Onto - DO - is_a, mapping(umls->do),(omim->do)'
    URL = "https://raw.githubusercontent.com/DiseaseOntology/HumanDiseaseOntology/master/src/ontology/doid.obo"
    OFILE_NAME = "DO_ontology.obo"
    QUADRUPLES = [('id', ' ', 1, 'ID'),
                  ('alt_id', ' ', 1, 'ALT_ID'),
                  ('is_a', ' ', 1, 'IS_A'),
                  ('xref: UMLS_CUI:', ':', 2, 'UMLS'),  # mapping umls
                  ('xref: OMIM:', ' ', 1, 'OMIM')]  # mapping omim
    DB_TYPE = DbType.DB_ONTO_DO

    def __init__(self):
        super().__init__(url=DbMetaOntoDo.URL,
                         ofile_name=DbMetaOntoDo.OFILE_NAME,
                         dbType=DbMetaOntoDo.DB_TYPE)



from openbiolink.graph_creation.file_reader.csvReader import CsvReader
from openbiolink.graph_creation.file_reader.edge import *
from openbiolink.graph_creation.file_reader.fileReader import FileReader
from openbiolink.graph_creation.file_reader.mapping import *
from openbiolink.graph_creation.file_reader.oboReader import OboReader
from openbiolink.graph_creation.file_reader.onto import *
from openbiolink.graph_creation.file_reader.postgresDumpReader import PostgresDumpReader
